% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% This function generates the deep features of a given image          %
%                                                                     %
%                                                                     %
% minCircleRadius   : The size of the smallest circular object        %
%                     (tissue component) to be located                %
% winSize           : The size of a salient subregion                 %
% K                 : The number of labels used for quantizing the    %
%                     salient subregions                              %
%                                                                     %
%                                                                     %
% weightMatFile     : A file containing the weights of RBMs           %
%                     the bias vector names should be B1, B2, ... and %
%                     the weight matrix names should be W1, W2, ...   %
%                     if you want to change the number of RBMs, you   %
%                     should modify the code given in Lines 22-29 in  %
%                     the assignLabels2SalientRegions function        %
% V                 : The clustering vectors                          %
%                                                                     %
%                                                                     %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
function deepFeatures = extractImageFeatures(imageFilename, weightMatFilename, V, minCircleRadius, winSize, K)

im = imread(imageFilename);
circleFilename = 'im_circles';
locateTissueCircles(im, 'H&E', 1, minCircleRadius, minCircleRadius, -1, circleFilename, 0);
salientRegionLabels = assignLabels2SalientRegions(im, circleFilename, winSize, weightMatFilename, V);
deepFeatures = extractHistogram(salientRegionLabels, K);
system('rm im_circles');

end
% --------------------------------------------------------------------%
% --------------------------------------------------------------------%
% --------------------------------------------------------------------%
function features = extractHistogram(salientRegionLabels, K)

features = zeros(1, K);
regNo = size(salientRegionLabels, 1);
tot = 0;
for i = 1 : regNo
    cL = salientRegionLabels(i);
    if cL > 0
        features(cL) = features(cL) + 1;
        tot = tot + 1;
    end
end
features = features / tot;

end
% --------------------------------------------------------------------%
% --------------------------------------------------------------------%
% --------------------------------------------------------------------%
