#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "matrix.h"
#include "circleFit.h"

/*************************************************************/
/*************************************************************/
/*************************************************************/
CIRCLE_BND *createCircularBoundaries(int maxRadius){
    CIRCLE_BND *B = (CIRCLE_BND *)calloc(maxRadius,sizeof(CIRCLE_BND));
    MATRIX M = allocateMatrix(maxRadius,maxRadius);
    int i, j, k, cnt;
    
    initializeMatrix(&M,-1);
    
    M.data[0][0] = 0;
    B[0].N = 1;
    B[0].r = 0;
    B[0].x = (int *)calloc(1,sizeof(int));
    B[0].y = (int *)calloc(1,sizeof(int));
    B[0].x[0] = 0;
    B[0].y[0] = 0;
    
    for (k = 1; k < maxRadius; k++){
        B[k].r = k;
        B[k].N = 0;
        B[k].x = B[k].y = NULL;
        for (i = 0; i <= k; i++)
            for (j = 0; j <= k; j++){
                if (M.data[i][j] != -1)
                    continue;
                if (i*i + j*j <= k*k){
                    M.data[i][j] = k;
                    B[k].N += 1;
                }
            }
        if (B[k].N == 0)
            continue;
        B[k].x = (int *)calloc(B[k].N,sizeof(int));
        B[k].y = (int *)calloc(B[k].N,sizeof(int));
        cnt = 0;
        for (i = 0; i <= k; i++)
            for (j = 0; j <= k; j++)
                if (M.data[i][j] == k){
                    B[k].x[cnt] = i;
                    B[k].y[cnt] = j;
                    cnt++;
                }
    }
    freeMatrix(M);
    return B;
}
void freeCircularBoundaries(CIRCLE_BND *B, int maxRadius){
    int i;
    
    for (i = 0; i < maxRadius; i++){
        free(B[i].x);
        free(B[i].y);
    }
    free(B);
}
/*************************************************************/
/*************************************************************/
/*************************************************************/
int findMaxPossibleCircleRadius(MATRIX map, int label, MATRIX *res){
	int i, j, d, maxR = 0;
	
	*res = allocateMatrix(map.row,map.column);
	for (i = 0; i < map.row; i++)
		for (j = 0; j < map.column; j++){
			if (map.data[i][j] != label)
				res->data[i][j] = -1;
			else {
				d = 1;
				while(1) {
					if ((i-d < 0)			|| (map.data[i-d][j] != label))		break;
					if ((i+d >= map.row)	|| (map.data[i+d][j] != label))		break;
					if ((j-d < 0)			|| (map.data[i][j-d] != label))		break;
					if ((j+d >= map.column)	|| (map.data[i][j+d] != label))		break;
					d++;
				}
				res->data[i][j] = d - 1;
				if (maxR < d - 1)
					maxR = d - 1;
			}
		}
	return maxR;
}
void findExactCircleRadius(MATRIX *res, CIRCLE_BND *cbnd, int maxR){
	int x, y, r, i, k, cx, cy;
	
	for (x = 0; x < res->row; x++)
		for (y = 0; y < res->column; y++)
			if (res->data[x][y] > 1){
				r = res->data[x][y];
				
				for (k = 2; k <= r; k++){
					for (i = 0; i < cbnd[k].N; i++){
						cx = cbnd[k].x[i];
						cy = cbnd[k].y[i];
						
						if ((res->data[x+cx][y+cy] == -1) || (res->data[x+cx][y-cy] == -1) ||
							(res->data[x-cx][y+cy] == -1) || (res->data[x-cx][y-cy] == -1))
							break;
					}
					if (i < cbnd[k].N)
						break;
				}
				res->data[x][y] = k - 1;
			}
}
void locateOneCircle(MATRIX *radii, CIRCLE_BND *cbnd, int x, int y, int R){
	int i, j, cx, cy, mx, my, r;
	int minx, miny, maxx, maxy;
	
	for (i = 0; i <= R; i++)
		for (j = 0; j < cbnd[i].N; j++){
			cx = cbnd[i].x[j];
			cy = cbnd[i].y[j];
			
			radii->data[x+cx][y+cy] = radii->data[x+cx][y-cy] = -1;
			radii->data[x-cx][y+cy] = radii->data[x-cx][y-cy] = -1;
		}
	minx = x - 2 * R;		if (minx < 0)					minx = 0;
	miny = y - 2 * R;		if (miny < 0)					miny = 0;
	maxx = x + 2 * R;		if (maxx >= radii->row)			maxx = radii->row - 1;
	maxy = y + 2 * R;		if (maxy >= radii->column)		maxy = radii->column - 1;
	
	for (mx = minx; mx <= maxx; mx++)
		for (my = miny; my <= maxy; my++){
			if (radii->data[mx][my] == -1)
				continue;
			
			r = radii->data[mx][my];
			for (i = 0; i <= r; i++){
				for (j = 0; j < cbnd[i].N; j++){
					cx = cbnd[i].x[j];
					cy = cbnd[i].y[j];
					
					if ((radii->data[mx+cx][my+cy] == -1) || (radii->data[mx+cx][my-cy] == -1) ||
						(radii->data[mx-cx][my+cy] == -1) || (radii->data[mx-cx][my-cy] == -1))
						break;
				}
				if (j < cbnd[i].N)
					break;
			}
			radii->data[mx][my] = i - 1;
		}
}
MATRIX locateCircles(MATRIX map, int compLabel, int minRadius, int maxRadius){
	MATRIX currR;
	int maxR = findMaxPossibleCircleRadius(map,compLabel,&currR) + 1;
	CIRCLE_BND *cbnd = createCircularBoundaries(maxR);
	int i, x, y, cno = 100, cnt;
	MATRIX circles = allocateMatrix(cno,4);
	
	findExactCircleRadius(&currR,cbnd,maxR);
    if ((maxRadius > 0) && (maxR > maxRadius + 1))
        maxR = maxRadius + 1;
	
	cnt = 0;
	for (i = maxR - 1; i >= minRadius; i--){
		for (x = 0; x < currR.row; x++)
			for (y = 0; y < currR.column; y++)
				if (currR.data[x][y] == i){
					locateOneCircle(&currR,cbnd,x,y,i);
					
					if (cnt >= cno){
						cno *= 2;
						reallocateMatrix(&circles,cno,4);
					}
					circles.data[cnt][0] = x;
					circles.data[cnt][1] = y;
					circles.data[cnt][2] = i;
					circles.data[cnt][3] = compLabel;
					cnt++;
				}
	}
	if (cnt == 0){
		freeMatrix(circles);
		circles.row = 0;
		circles.column = 0;
		circles.data = NULL;	
	}
	else
		reallocateMatrix(&circles,cnt,4);
	
	freeMatrix(currR);
	freeCircularBoundaries(cbnd,maxR);
	return circles;
}

